
#include "TArtSimpleFunction.hh"
#include "TArtHistManager.hh"
#include "TArtCanvasManager.hh"
#include "TArtGlobal.hh"
#include "TArtExec.hh"

#include "TH1.h"
#include "TH1D.h"
#include "TH2.h"
#include "TH2D.h"
#include "TH3.h"
#include "TH3D.h"
#include "TProfile.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TList.h"
#include "TCutG.h"
#include "TPad.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TKey.h"
#include "TClass.h"

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <algorithm>
#include <fstream>
#include <vector>

TH1* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title,
					   Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH1* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH1*>(obj);
  }else{
    hist = new TH1D(name, title, nbinsx, xlow, xup);
  }
  hist->Fill(valx);

  return hist;
}

TH2* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title,
					   Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup,
					   Double_t valy, Int_t nbinsy, Double_t ylow, Double_t yup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH2* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH2*>(obj);
  }else{
    hist = new TH2D(name, title, nbinsx, xlow, xup, nbinsy, ylow, yup);
  }
  hist->Fill(valx, valy);

  return hist;
}

TH3* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title,
					   Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup,
					   Double_t valy, Int_t nbinsy, Double_t ylow, Double_t yup,
					   Double_t valz, Int_t nbinsz, Double_t zlow, Double_t zup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH3* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH3*>(obj);
  }else{
    hist = new TH3D(name, title, nbinsx, xlow, xup, nbinsy, ylow, yup, nbinsz, zlow, zup);
  }
  hist->Fill(valx, valy, valz);

  return hist;
}

TH1* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title, Int_t size,
					   Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH1* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH1*>(obj);
  }else{
    hist = new TH1D(name, title, nbinsx, xlow, xup);
  }
  for(int i=0; i<size; ++i) hist->Fill(valx[i]);

  return hist;
}

TH2* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title, Int_t size,
					   Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup,
					   Double_t* valy, Int_t nbinsy, Double_t ylow, Double_t yup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH2* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH2*>(obj);
  }else{
    hist = new TH2D(name, title, nbinsx, xlow, xup, nbinsy, ylow, yup);
  }
  for(int i=0; i<size; ++i) hist->Fill(valx[i], valy[i]);

  return hist;
}

TH3* TArtSimpleFunction::CreateAndFillHist(const char* name, const char* title, Int_t size,
					   Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup,
					   Double_t* valy, Int_t nbinsy, Double_t ylow, Double_t yup,
					   Double_t* valz, Int_t nbinsz, Double_t zlow, Double_t zup)
{
  if(!gDirectory) return 0;
  TObject* obj = gDirectory->FindObject(name);
  TH3* hist;
  if(obj && obj->InheritsFrom(TH1::Class())){
    hist = static_cast<TH3*>(obj);
  }else{
    hist = new TH3D(name, title, nbinsx, xlow, xup, nbinsy, ylow, yup, nbinsz, zlow, zup);
  }
  for(int i=0; i<size; ++i) hist->Fill(valx[i], valy[i], valz[i]);

  return hist;
}

void TArtSimpleFunction::ReadObjectFromFile(char* filename)
{
  if(!gDirectory) return;
  TDirectory* lastdirectory = gDirectory;

  TFile* file = TFile::Open(filename, "READ");
  if(!file || file->IsZombie()) return;
  TList* list = file->GetListOfKeys();
  
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    if(obj && obj->InheritsFrom(TKey::Class())){
      TKey* key = static_cast<TKey*>(obj);
      obj = key->ReadObj();
      if(obj && obj->InheritsFrom(TNamed::Class())){
	TNamed* named = static_cast<TNamed*>(obj);
	std::string name = TArtGlobal::IncrementName(named->GetName(), lastdirectory);
	named->SetName(name.c_str());
      }
      file->Remove(obj);
      ROOT::DirAutoAdd_t func = obj->IsA()->GetDirectoryAutoAdd();
      if(func){
	func(obj, lastdirectory);
      }else{
	lastdirectory->Append(obj);
      }
    }
    objlink = objlink->Next();
  }
  file->Close();

  lastdirectory->cd();
}

void TArtSimpleFunction::WriteObjectToFile(char* filename)
{
  if(!gDirectory) return;
  TDirectory* lastdirectory = gDirectory;  

  TFile* file = TFile::Open(filename, "NEW");
  if(!file || file->IsZombie()) return;

  TList* list = lastdirectory->GetList();
  TObjLink* objlink = list->FirstLink();
  while(objlink){
    TObject* obj = objlink->GetObject();
    obj->Write(0, TObject::kSingleKey);
    objlink = objlink->Next();
  }
  file->Close();

  lastdirectory->cd();
}

TH1* TArtSimpleFunction::WriteHistToText(char* filename)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  std::ofstream ofs;
  ofs.open(filename, std::ios::in);
  if(ofs){
    std::cout << "file exists" << std::endl;
    return 0;
  }
  ofs.close();
  ofs.open(filename, std::ios::out);
  if(!ofs){
    std::cout << "cannot open file" << std::endl;
    return 0;
  }

  ofs << "# " << hist->GetName() << " " << hist->GetTitle() << std::endl;

  Int_t dimension = hist->GetDimension();
  if(dimension == 1){
    for(Int_t binx=1; binx<=hist->GetNbinsX(); ++binx){
      Double_t bincontent = hist->GetBinContent(binx);
      Double_t x = hist->GetXaxis()->GetBinCenter(binx);
      ofs << bincontent << " " << x << std::endl;
    }
  }else if(dimension == 2){
    for(Int_t biny=1; biny<=hist->GetNbinsY(); ++biny){
      for(Int_t binx=1; binx<=hist->GetNbinsX(); ++binx){
	Double_t bincontent = hist->GetBinContent(binx, biny);
	Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	ofs << bincontent << " " << x << " " << y << std::endl;
      }
    }
  }else if(dimension == 3){
    for(Int_t binz=1; binz<=hist->GetNbinsZ(); ++binz){
      for(Int_t biny=1; biny<=hist->GetNbinsY(); ++biny){
	for(Int_t binx=1; binx<=hist->GetNbinsX(); ++binx){
	  Double_t bincontent = hist->GetBinContent(binx, biny, binz);
	  Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	  Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	  Double_t z = hist->GetZaxis()->GetBinCenter(binz);
	  ofs << bincontent << " " << x << " " << y << " " << z << std::endl;
	}
      }
    }
  }

  return hist;
}

TH1* TArtSimpleFunction::CreateHistFromText(char* filename)
{
  std::ifstream ifs;
  ifs.open(filename, std::ios::in);
  if(!ifs){
    std::cout << "cannot open file" << std::endl;
    return 0;
  }

  std::vector<std::vector<double> > xy;
  while(1){
    std::string str;
    std::getline(ifs, str);
    if(ifs.eof()) break;
    if(str.find("#") != std::string::npos) continue;
    std::istringstream iss(str);

    std::vector<double> x;
    double xtemp;
    while(iss >> xtemp) x.push_back(xtemp);
    xy.push_back(x);
  }

  if(xy.empty()) return 0;

  Int_t dimension = (xy[0]).size()-1;

  TH1* hist = 0;
  std::string name(filename);
  name = TArtGlobal::IncrementName(name);  

  Double_t entries = 0;
  if(dimension == 1){
    Int_t nbins = xy.size();
    Double_t xmin = xy[0][1];
    Double_t xmax = xy[xy.size()-1][1];
    Double_t dx = (xmax - xmin)/(nbins-1);

    hist = new TH1D(name.c_str(), "create from text", nbins, xmin-dx/2, xmax+dx/2);
    for(Int_t binx=1; binx<=xy.size(); ++binx){
      Double_t bincontent = xy[binx-1][0];
      hist->SetBinContent(binx, bincontent);
      entries += bincontent;
    }
  }else if(dimension == 2){
//    Int_t nbins = xy.size();
//    Double_t xmin = xy[0][1];
//    Double_t xmax = xy[xy.size()-1][1];
//    Double_t dx = (xmax - xmin)/(nbins-1);
//    Double_t ymin = xy[0][2];
//    Double_t ymax = xy[xy.size()-1][2];
//    Double_t dy = (ymax - ymin)/(nbins-1);
//
//    hist = new TH2D(name.c_str(), "create from text",
//		    nbinsx, xmin-dx/2, xmax+dx/2,
//		    nbinsy, ymin-dy/2, ymax+dy/2);
//    for(Int_t biny=1; biny<=xy.size(); ++biny){
//      for(Int_t binx=1; binx<=xy.size(); ++binx){
//	entries += bincontent;
//      }
//    }
    std::cout << "not yet" << std::endl;
    return 0;
  }else if(dimension == 3){
    std::cout << "not yet" << std::endl;
    return 0;
  }else{
    std::cout << "file format is not appropriate" << std::endl;    
    return 0;
  }

  hist->SetEntries(entries);

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TGraph* TArtSimpleFunction::CreateGraphFromText(char* filename)
{
  std::ifstream ifs;
  ifs.open(filename, std::ios::in);
  if(!ifs){
    std::cout << "cannot open file" << std::endl;
    return 0;
  }

  std::vector<std::vector<double> > xy;
  while(1){
    std::string str;
    std::getline(ifs, str);
    if(ifs.eof()) break;
    if(str.find("#") != std::string::npos) continue;
    std::istringstream iss(str);

    std::vector<double> x;
    double xtemp;
    while(iss >> xtemp) x.push_back(xtemp);
    xy.push_back(x);
  }

  if(xy.empty()) return 0;

  Int_t dimension = (xy[0]).size();
  if(dimension != 2){
    std::cout << "file format is not appropriate" << std::endl;    
    return 0;
  }

  std::vector<double> x,y;
  for(int i=0; i<xy.size(); ++i){
    x.push_back(xy[i][0]);
    y.push_back(xy[i][1]);
  }

  if(!gDirectory) return 0;
  TList* list;
  TObject* obj = gDirectory->FindObject("ListOfGraphs");
  if(!obj){
    list = new TList;
    list->SetName("ListOfGraphs");
    list->SetOwner();
    gDirectory->Append(list);
  }else{
    if(obj->InheritsFrom(TList::Class())){
      list = static_cast<TList*>(obj);
    }else{
      std::cout << "cannot get list of graphs" << std::endl;
      return 0;
    }
  }

  std::string name("Graph");
  name = TArtGlobal::IncrementName(name, list);
  TGraph* graph = new TGraph(x.size(), &(*x.begin()), &(*y.begin()));
  graph->SetName(name.c_str());
  graph->SetTitle("create from text");

  list->Add(graph);
  graph->Draw("apl");

  return graph;
}

void TArtSimpleFunction::StatusX(Int_t binx1, Int_t binx2)
{
  TH1* hist = gH1;
  if(!hist) return;

  if(hist->GetDimension() != 1){
    std::cout << " not 1d hist" << std::endl;
    return;
  }

  Int_t entries = static_cast<Int_t>(hist->GetEntries());
  Int_t contents = static_cast<Int_t>(hist->Integral(binx1, binx2));
  Double_t area = hist->Integral(binx1, binx2, "width");

  std::cout << std::endl;
  std::cout << " Entries:" << entries << std::endl;
  std::cout << " Contents:" << contents << std::endl;
  std::cout << " Ratio:" << 100*contents/entries << "%" << std::endl;
  std::cout << " Area:" << area << std::endl;
  std::cout << std::endl;
}

void TArtSimpleFunction::GStatusX()
{
  TH1* hist = gH1;
  if(!hist) return;

  if(hist->GetDimension() != 1){
    std::cout << " not 1d hist" << std::endl;
    return;
  }

  Double_t xmin = TArtExec::GXValue();
  Double_t xmax = TArtExec::GXValue();
  Int_t binxmin = hist->FindBin(xmin);
  Int_t binxmax = hist->FindBin(xmax);

  if(binxmin>binxmax) std::swap(binxmin,binxmax);

  StatusX(binxmin, binxmax);
}

TH2* TArtSimpleFunction::CutGHist()
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);

  if(!gPad) return 0;
  TCutG* cutg = static_cast<TCutG*>(gPad->WaitPrimitive("CUTG","CutG"));
  if(!cutg) return 0;
  cutg->SetName("cutg_cpro");

  std::string name(hist2->GetName());
  name += "_cpro";
  name = TArtGlobal::IncrementName(name);
  TObject* obj = hist2->Clone(name.c_str());
  if(!obj) return 0;
  TH2* hist2cpro = static_cast<TH2*>(obj);
  hist2cpro->Reset();

  //0:under flow, nbin+1:over flow
  Double_t entries = 0;
  for(Int_t biny=0; biny<hist2->GetNbinsY()+2; ++biny){
    for(Int_t binx=0; binx<hist2->GetNbinsX()+2; ++binx){
      Double_t x = hist2->GetXaxis()->GetBinCenter(binx);
      Double_t y = hist2->GetYaxis()->GetBinCenter(biny);
      if(cutg->IsInside(x,y)){
	Double_t bincontent = hist2->GetBinContent(binx, biny);
	hist2cpro->SetBinContent(binx, biny, bincontent);
	entries += bincontent;
      }
    }
  }
  hist2cpro->SetEntries(entries);

  if(!TArtHistManager::InsertAfterHist(hist2, hist2cpro)){
    delete hist2cpro;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist2cpro);

  return hist2cpro;
}

TFitResultPtr TArtSimpleFunction::Fit(const char* formula, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  TH1* hist = gH1;
  if(hist){
    return hist->Fit(formula, option, goption, xmin, xmax);
  }else{
   return TFitResultPtr();
  }
}

TFitResultPtr TArtSimpleFunction::FitPolynomial(int n, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  std::ostringstream os;
  os << n;
  std::string pol("pol" + os.str());

  return Fit(pol.c_str(), option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitExponential(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("expo", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitGaussian(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("gaus", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitLandau(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("landau", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::GFitPolynomial(int n, Option_t* option, Option_t* goption)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t xmin = TArtExec::GXValue();
  Double_t xmax = TArtExec::GXValue();
  if(xmin>xmax) std::swap(xmin,xmax);

  return FitPolynomial(n, option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::GFitExponential(Option_t* option, Option_t* goption)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t xmin = TArtExec::GXValue();
  Double_t xmax = TArtExec::GXValue();
  if(xmin>xmax) std::swap(xmin,xmax);

  return FitExponential(option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::GFitGaussian(Option_t* option, Option_t* goption)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t xmin = TArtExec::GXValue();
  Double_t xmax = TArtExec::GXValue();
  if(xmin>xmax) std::swap(xmin,xmax);

  return FitGaussian(option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::GFitLandau(Option_t* option, Option_t* goption)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t xmin = TArtExec::GXValue();
  Double_t xmax = TArtExec::GXValue();
  if(xmin>xmax) std::swap(xmin,xmax);

  return FitLandau(option, goption, xmin, xmax);
}

TProfile* TArtSimpleFunction::ProfileX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_pfx";
  name = TArtGlobal::IncrementName(name);
  TProfile* prof = hist2->ProfileX(name.c_str(), firstybin, lastybin, option);
  prof->SetMinimum(hist2->GetYaxis()->GetXmin());
  prof->SetMaximum(hist2->GetYaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  if(!TArtHistManager::InsertAfterHist(hist2, prof)){
    delete prof;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TProfile* TArtSimpleFunction::ProfileY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_pfy";
  name = TArtGlobal::IncrementName(name);
  TProfile* prof = hist2->ProfileY(name.c_str(), firstxbin, lastxbin, option);
  prof->SetMinimum(hist2->GetXaxis()->GetXmin());
  prof->SetMaximum(hist2->GetXaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  if(!TArtHistManager::InsertAfterHist(hist2, prof)){
    delete prof;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TH1D* TArtSimpleFunction::ProjectionX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_px";
  name = TArtGlobal::IncrementName(name);
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    delete histd;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::ProjectionY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_py";
  name = TArtGlobal::IncrementName(name);
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    delete histd;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1* TArtSimpleFunction::SliceX(Int_t num)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(num < 1) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  TH1* histlast = static_cast<TH1*>(hist2);
  TH1D* histd;
  Int_t binmin = hist->GetYaxis()->GetFirst();
  Int_t binmax = hist->GetYaxis()->GetLast();
  for(int i=0; i<num; ++i){
    Int_t firstybin = binmin+(binmax-binmin)*i/num;
    Int_t lastybin = binmin+(binmax-binmin)*(i+1)/num;

    std::ostringstream oss;
    oss << hist2->GetName() << "_slx" << i;
    std::string namesly(oss.str());
    namesly = TArtGlobal::IncrementName(namesly);

    histd = hist2->ProjectionX(namesly.c_str(), firstybin, lastybin, "o");

    if(!TArtHistManager::InsertAfterHist(histlast, histd)){
      delete histd;
      return 0;
    }

    if(i == 0){
      TArtCanvasManager::cdNext();
      TArtHistManager::DrawHist(histd);
    }

    histlast = histd;
  }

  return histd;
}

TH1* TArtSimpleFunction::SliceY(Int_t num)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(num < 1) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  TH1* histlast = static_cast<TH1*>(hist2);
  TH1D* histd;
  Int_t binmin = hist->GetXaxis()->GetFirst();
  Int_t binmax = hist->GetXaxis()->GetLast();
  for(int i=0; i<num; ++i){
    Int_t firstxbin = binmin+(binmax-binmin+1)*i/num;
    Int_t lastxbin = binmin+(binmax-binmin+1)*(i+1)/num-1;

    std::ostringstream oss;
    oss << hist2->GetName() << "_sly" << i;
    std::string namesly(oss.str());
    namesly = TArtGlobal::IncrementName(namesly);

    histd = hist2->ProjectionY(namesly.c_str(), firstxbin, lastxbin, "o");

    if(!TArtHistManager::InsertAfterHist(histlast, histd)){
      delete histd;
      return 0;
    }

    if(i == 0){
      TArtCanvasManager::cdNext();
      TArtHistManager::DrawHist(histd);
    }

    histlast = histd;
  }

  return histd;
}

TH1D* TArtSimpleFunction::BanY(double firsty, double lasty, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_bny";
  name = TArtGlobal::IncrementName(name);
  Int_t firstybin = hist2->GetYaxis()->FindBin(firsty);
  Int_t lastybin  = hist2->GetYaxis()->FindBin(lasty);
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    delete histd;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::BanX(double firstx, double lastx, Option_t* option)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = static_cast<TH2*>(hist);
  std::string name(hist2->GetName());
  name += "_bnx";
  name = TArtGlobal::IncrementName(name);
  Int_t firstxbin = hist2->GetXaxis()->FindBin(firstx);
  Int_t lastxbin  = hist2->GetXaxis()->FindBin(lastx);
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  if(!TArtHistManager::InsertAfterHist(hist2, histd)){
    delete histd;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1* TArtSimpleFunction::Range(double firstx, double lastx, double firsty, double lasty, double firstz, double lastz)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(hist->GetDimension() == 1){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
  }else if(hist->GetDimension() == 2){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
    hist->GetYaxis()->SetRangeUser(firsty, lasty);
  }else if(hist->GetDimension() == 3){
    hist->GetXaxis()->SetRangeUser(firstx, lastx);
    hist->GetYaxis()->SetRangeUser(firsty, lasty);
    hist->GetZaxis()->SetRangeUser(firstz, lastz);
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeX(double firstx, double lastx)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  hist->GetXaxis()->SetRangeUser(firstx, lastx);

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeY(double firsty, double lasty)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() == 1) return 0;
  
  hist->GetYaxis()->SetRangeUser(firsty, lasty);

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::RangeZ(double firstz, double lastz)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 3) return 0;

  hist->GetZaxis()->SetRangeUser(firstz, lastz);

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(hist);

  return hist;
}

TH1* TArtSimpleFunction::GRange()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(hist->GetDimension() == 1){
    return GRangeX();
  }else if(hist->GetDimension() == 2){
    return GRangeXY();    
  }else{
    std::cout << " not 1d or 2d hist" << std::endl;
    return 0;
  }
}

TH1* TArtSimpleFunction::GRangeX()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t x1 = TArtExec::GXValue();
  Double_t x2 = TArtExec::GXValue();
  if(x1>x2) std::swap(x1,x2);

  return RangeX(x1,x2);
}

TH1* TArtSimpleFunction::GRangeY()
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() == 1){
    std::cout << " not 2d nor 3d hist" << std::endl;
    return 0;
  }

  Double_t y1 = TArtExec::GYValue();
  Double_t y2 = TArtExec::GYValue();
  if(y1>y2) std::swap(y1,y2);

  return RangeY(y1,y2);
}

TH1* TArtSimpleFunction::GRangeXY()
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t x1,y1;
  Double_t x2,y2;
  TArtExec::GXYValue(x1,y1);
  TArtExec::GXYValue(x2,y2);
  if(x1>x2) std::swap(x1,x2);
  if(y1>y2) std::swap(y1,y2);

  return Range(x1,x2,y1,y2);
}

TH1* TArtSimpleFunction::Blow(double firstx, double lastx, double firsty, double lasty, double firstz, double lastz)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(firstx>lastx){
    firstx = gH1->GetXaxis()->GetXmin();
    lastx = gH1->GetXaxis()->GetXmax();
  }
  if(firsty>lasty){
    firsty = gH1->GetYaxis()->GetXmin();
    lasty = gH1->GetYaxis()->GetXmax();
  }
  if(firstz>lastz){
    firstz = gH1->GetZaxis()->GetXmin();
    lastz = gH1->GetZaxis()->GetXmax();
  }

  std::string name(hist->GetName());
  name += "_blow";
  name = TArtGlobal::IncrementName(name);
  TObject* obj = hist->Clone(name.c_str());
  if(!obj) return 0;
  TH1* histblow = static_cast<TH1*>(obj);
  histblow->Reset();
  Double_t entries = 0;
  Int_t dimension = histblow->GetDimension();
  if(dimension == 1){
    for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
      Double_t x = hist->GetXaxis()->GetBinCenter(binx);
      if(x >= firstx && x <= lastx){
	Double_t bincontent = hist->GetBinContent(binx);
	histblow->SetBinContent(binx, bincontent);
	entries += bincontent;
      }
    }
  }else if(dimension == 2){
    for(Int_t biny=0; biny<hist->GetNbinsY()+2; ++biny){
      for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
	Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	if((x >= firstx && x <= lastx) &&
	   (y >= firsty && y <= lasty)){
	  Double_t bincontent = hist->GetBinContent(binx, biny);
	  histblow->SetBinContent(binx, biny, bincontent);
	  entries += bincontent;
	}
      }
    }
  }else if(dimension == 3){
    for(Int_t binz=0; binz<hist->GetNbinsZ()+2; ++binz){
      for(Int_t biny=0; biny<hist->GetNbinsY()+2; ++biny){
	for(Int_t binx=0; binx<hist->GetNbinsX()+2; ++binx){
	  Double_t x = hist->GetXaxis()->GetBinCenter(binx);
	  Double_t y = hist->GetYaxis()->GetBinCenter(biny);
	  Double_t z = hist->GetZaxis()->GetBinCenter(binz);
	  if((x >= firstx && x <= lastx) &&
	     (y >= firsty && y <= lasty) &&
	     (z >= firstz && z <= lastz)){
	    Double_t bincontent = hist->GetBinContent(binx, biny, binz);
	    histblow->SetBinContent(binx, biny, binz, bincontent);
	    entries += bincontent;
	  }
	}
      }
    }
  }
  histblow->SetEntries(entries);

  if(!TArtHistManager::InsertAfterHist(hist, histblow)){
    delete histblow;
    return 0;
  }

  if(TArtHistManager::SetHist(histblow)) Range(firstx, lastx, firsty, lasty, firstz, lastz);

  return histblow;
}

TH1* TArtSimpleFunction::BlowX(double firstx, double lastx)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  return Blow(firstx, lastx);
}

TH1* TArtSimpleFunction::BlowY(double firsty, double lasty)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  return Blow(gH1->GetXaxis()->GetXmin(), gH1->GetXaxis()->GetXmax(),
	      firsty, lasty);
}

TH1* TArtSimpleFunction::BlowZ(double firstz, double lastz)
{
  TH1* hist = gH1;
  if(!hist) return 0;

  return Blow(gH1->GetXaxis()->GetXmin(), gH1->GetXaxis()->GetXmax(),
	      gH1->GetYaxis()->GetXmin(), gH1->GetYaxis()->GetXmax(),
	      firstz, lastz);
}

TH1* TArtSimpleFunction::GBlow()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(hist->GetDimension() == 1){
    return GBlowX();
  }else if(hist->GetDimension() == 2){
    return GBlowXY();
  }else{
    std::cout << " not 1d or 2d hist" << std::endl;
    return 0;
  }
}

TH1* TArtSimpleFunction::GBlowX()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  Double_t x1 = TArtExec::GXValue();
  Double_t x2 = TArtExec::GXValue();
  if(x1>x2) std::swap(x1,x2);

  return Blow(x1,x2);
}

TH1* TArtSimpleFunction::GBlowY()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t y1 = TArtExec::GYValue();
  Double_t y2 = TArtExec::GYValue();
  if(y1>y2) std::swap(y1,y2);

  return Blow(y1,y2);
}

TH1* TArtSimpleFunction::GBlowXY()
{
  TH1* hist = gH1;
  if(!hist) return 0;

  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  Double_t x1,y1;
  Double_t x2,y2;
  TArtExec::GXYValue(x1,y1);
  TArtExec::GXYValue(x2,y2);
  if(x1>x2) std::swap(x1,x2);
  if(y1>y2) std::swap(y1,y2);

  return Blow(x1,x2,y1,y2);
}

TH1* TArtSimpleFunction::MinMax(double min, double max)
{
  if(min >= max) return 0;

  TH1* hist = gH1;
  if(!hist) return 0;

  std::string name = std::string(hist->GetName()) + "_mami";
  name = TArtGlobal::IncrementName(name);  
  TObject* obj = hist->Clone(name.c_str());
  if(!obj) return 0;
  TH1* histmami = static_cast<TH1*>(obj);
  histmami->SetMinimum(min);
  histmami->SetMaximum(max);

  if(!TArtHistManager::InsertAfterHist(hist, histmami)){
    delete histmami;
    return 0;
  }

  TArtCanvasManager::cdNext();
  TArtHistManager::DrawHist(histmami);

  return histmami;
}
